package de.bsd.pmdrules;

import java.util.List;

import net.sourceforge.pmd.Rule;
import net.sourceforge.pmd.RuleContext;
import net.sourceforge.pmd.ast.ASTBlockStatement;
import net.sourceforge.pmd.ast.ASTIfStatement;
import net.sourceforge.pmd.ast.ASTName;
import net.sourceforge.pmd.ast.ASTPrimaryExpression;
import net.sourceforge.pmd.ast.ASTPrimaryPrefix;
import net.sourceforge.pmd.ast.ASTPrimarySuffix;
import net.sourceforge.pmd.ast.SimpleNode;
import net.sourceforge.pmd.rules.optimization.AbstractOptimizationRule;

/**
 * Check that log.debug and log.trace statements are guarded by some
 * log.isDebugEnabled() or log.isTraceEnabled() checks.
 * @author hwr@pilhuhn.de
 *
 */
public class GuardDebugLogger extends AbstractOptimizationRule implements Rule {

	public Object visit(ASTPrimaryExpression node, Object data) {

		if (node!=null) {
			SimpleNode pprefix = (SimpleNode) node.jjtGetChild(0);
			if (pprefix == null || !(pprefix instanceof ASTPrimaryPrefix)) {
				return super.visit(node, data);
			}
				
			ASTPrimaryPrefix p2 = (ASTPrimaryPrefix) pprefix;
			ASTName name = p2.getFirstChildOfType(ASTName.class);
			if (name!=null) {
				String image = name.getImage();
				if (image.endsWith(".debug") || image.endsWith(".trace")) {
					boolean isTrace = image.endsWith(".trace");
					// TODO check for type 
					SimpleNode parent1 = (SimpleNode) node.getNthParent(3);
					boolean guardFound = false;
					if (parent1 instanceof ASTIfStatement) {
						guardFound = checkForGuard((ASTIfStatement) parent1, isTrace);
					}
					else if (parent1 instanceof ASTBlockStatement) {
						SimpleNode parent2 = (SimpleNode)node.getNthParent(6);
						if (parent2 instanceof ASTIfStatement) {
							guardFound = checkForGuard((ASTIfStatement) parent2, isTrace);
							
						}
					}
					if (!guardFound)
						addViolation(data, node);
				}
			}
		}
		return super.visit(node, data);
	}

	/**
	 * We stand on an if() check if it contains log.isDebugEnabled()
	 * @param stm
	 * @param isTrace true if log.trace() is used
	 * @return true if guard was found
	 */
	private boolean checkForGuard(ASTIfStatement stm, boolean isTrace) {
		
		ASTPrimaryPrefix pp = stm.getFirstChildOfType(ASTPrimaryPrefix.class);
		if (pp == null)
			return false;
		
		ASTName name = pp.getFirstChildOfType(ASTName.class);
		if (name == null)
			return false;
		
		String image = name.getImage();
		if (image==null)
			return false;
		
		if (image.endsWith(".isDebugEnabled") && !isTrace)
			return true;
		if (image.endsWith(".isTraceEnabled"))
			return true;
		
		return false;
	}
}
